@@ -1,144 +1,164 @@
 #
-# Copyright (c) 2002
-#          Philippe 'BooK' Bruhat <book@cpan.org>
-#          Dave Hoover            <dave@redsquirreldesign.com>
-#          Steffen Muller         <games-golf@steffen-mueller.net>
-#          Jonathan E. Paton      <jonathanpaton@yahoo.com>
-#          Jerome Quelin          <jquelin@cpan.org>
-#          Eugene Van der Pijll   <E.C.vanderPijll@phys.uu.nl>
-# All rights reserved.
-#
-# This program is free software; you can redistribute it and/or
-# modify it under the same terms as Perl itself.
-#
-# $Id: Golf.pm,v 1.32 2002/03/25 16:19:10 smueller Exp $
+# $Id: Golf.pm,v 1.38 2002/05/13 14:03:57 smueller Exp $
 #
 
 package Games::Golf;
 
 use 5.005;
 use strict;
-use vars qw/$VERSION/;
+local $^W = 1; # Enable warnings the old way
 
-# Modules we rely upon.
+use vars qw/ $VERSION $AUTOLOAD $subs /;
+BEGIN {
+    # add all registered accessors here
+    my @subs = split /\|/, $subs = 'file';
+    use subs @subs;
+}
+
+$VERSION = '0.14';
+
+# Modules we rely upon...
 use Carp;
 use Games::Golf::Entry;
 use Games::Golf::TestSuite;
 
-# Variables of the module. blah
-local $^W = 1;    # use warnings for perl < 5.6
-$VERSION = '0.13';
-
 =head1 NAME
 
-Games::Golf - Utilities to administer Perl Golf courses
+Games::Golf - Utilities to handle Perl Golf courses
 
 =head1 SYNOPSIS
 
   use Games::Golf;
-  my $golf = new Games::Golf 
-      ( referees => { "Bob the Referee" => 'bob@referee.org',
-                      "Arbiter Jim"     => 'jim@arbiter.net' },
-        holes    => [ qw( name1 name2 ) ],
-        deadline     => "2002.02.20.13.27.20",
-      );
+  my $golf = new Games::Golf( "tpr.glf" ); 
 
   $golf->read( "entries.dat" );
 
-  $golf->extract( "mbox" );
-
   $golf->test;
 
   $golf->dump( "entries.dat" );
 
-  $golf->print_report;
-
-  $golf->mail_report( 'fwp@perl.org' );
-
-
-  Or, interactive:
-
-  perl -MGames::Golf -e shell
-
-
 =head1 DESCRIPTION
 
-The game of Perl golf is becoming increasingly popular. Holes frequently
-appear on Perl Monks, the Perl Review has an ongoing Perl Golf colunm,
-and the Fun with Perl mailing-list has run several very successful 
-courses in
-the last few months, with great success.
+The game of Perl golf is becoming increasingly popular. Holes
+frequently appear on Perl Monks, the Perl Review has an ongoing Perl
+Golf column and a monthly tournament, the Fun with Perl mailing-list
+has run several very successful courses in the last few months, with
+great success. So much success, in fact, that the golf@perl.org
+mailing-list has been resurrected.
 
-Until now, judges have had to write extensive test programs to
-check and score the players' entries along with compiling the
-leaderboard by hand.
+Until now, judges have had to write extensive test programs to check
+and score the players' entries along with compiling the leaderboard by
+hand. On the other hand, golfers have to check their entries, and
+submit their entries themselves.
 
-This module aims at facilitating the administration of Perl golf courses
-by writing a simple configuration file and writing the test programs
-as .t files.
+This module aims at facilitating the administration of Perl golf
+courses by writing a simple configuration file for the referees, and
+providing scripts for both the referee and the player side.
 
-The included scripts will test a player's individual entries, doing all
-the hard scoring and testing for the judge. Methods to compute statistics
-and leaderboards are also provided.
+The included golfer script will test a player's individual entries
+(doing all the hard scoring and testing), as well as submitting the
+entries.
 
 The C<Games::Golf> object will handle a list of C<Games::Golf::Entry> 
-objects.
+objects, as well as a list of C<Games::Golf::TestSuite.pm> objects.
 
 =head2 CONSTRUCTOR
 
 =over 4
 
-=item new( [ARGS] )
+=item new( $file )
 
-Creates a new C<Games::Golf> object. Parameters are:
-
-  referees      A hash ref for the referees. The keys are the names
-                of the referees, the values are the email addresses.
-  deadline      Closing date of the game. Any format welcome.
-  holes         Array ref to the names of the holes.
-
-=back
+Creates a new C<Games::Golf> object. You are to provide the name of a
+configuration file that will describe the course (see L<"CONFIG FILE">
+for details about this file).
 
 =cut
+
 sub new {
-
     # Create the object, and bless it.
-    my $class    = shift;
-    my %args     = @_;
-
-    my $self     = {};
+    my $class = shift;
+    my $file  = shift || croak "Not enough parameters";
+    my $self     = 
+      { file     => $file,
+        entries  => {},
+        deadline => undef,
+        referees => {},
+        testers  => {}
+      };
     bless $self, $class;
 
-    # get attribute keys and defaults.
-    my %defaults = _defaults();
-
-    # set attributes to passed attributes or to defaults.
-    foreach my $attr ( keys %defaults ) {
-        $self->{$attr} = exists $args{$attr} ? $args{$attr} : $defaults{$attr};
-    }
-
-    # Create the testsuite for each hole.
-    foreach my $hole ( @{ $self->{holes} } ) {
-        # !!FIXME!! why t/ ?
-        $self->{testers}{$hole} = new Games::Golf::TestSuite( "t/$hole" );
-    }
+    $self->_parse_config_file();
 
     # Return the new object.
     return $self;
 }
 
+=back
+
+=head2 ACCESSORS
+
+=over 4
+
+=item hole_names(  )
+
+Return the names of the holes of the course.
+
+=cut
+
+sub hole_names {
+    my $self = shift;
+    return keys %{ $self->{testers} };
+}
+
+=back
+
+The following accessors are autoloaded.
+
+=over 4
+
+=item file(  )
+
+The configuration file of the course.
+
+=cut
+sub AUTOLOAD {
+    # we don't DESTROY
+    return if $AUTOLOAD =~ /::DESTROY/;
+
+    # fetch the attribute name
+    $AUTOLOAD =~ /.*::(\w+)/;
+    my $attr = $1;
+    # must be one of the registered subs (compile once)
+    if( $attr =~ /$subs/o ) {
+        no strict 'refs';
+
+        # create the method (but don't pollute other namespaces)
+        *{$AUTOLOAD} = sub {
+            my $self = shift;
+            @_ ? $self->{$attr} = shift: $self->{$attr};
+        };
+
+        # now do it
+        goto &{$AUTOLOAD};
+    }
+    # should we really die here?
+    croak "Undefined method $AUTOLOAD";
+}
+
+=back
+
 =head2 PUBLIC METHODS
 
 =over 4
 
-=item extract( "/path/to/mbox", ... )
+=item FIXME_extract( "/path/to/mbox", ... )
 
 Extracts solutions from a unix-style mbox. A cache mechanism allows
-the C<Golf::Game> object not to extract already extracted entries.
+the C<Games::Golf> object not to extract already extracted entries.
 
 =cut 
 
-sub extract {
+sub FIXME_extract {
     my $self = shift;
     my @new;
 
@@ -242,41 +262,50 @@ sub extract {
     return @new;
 }
 
-=item test()
+=item add( $entry )
 
-Tests the entries of the game. A cache mechanism (implemented in
-Games::Golf::TestSuite::run()) allows the Games::Golf object not to
+Store an entry in the C<Games::Golf> object. The paramater is a
+C<Games::Golf::Entry> object.
+
+Return the entry (either the new one, or the old one if the same entry
+was already in the cache).
+
+=cut
+
+sub add {
+    my ($self, $entry) = @_;
+
+    my $id = $entry->id();
+    $self->{entries}{$id} = $entry unless
+      exists $self->{entries}{$id};
+    return $self->{entries}{$id};
+}
+
+=item test( [ @holes ] )
+
+Tests the entries stored in the object. A cache mechanism (implemented
+in C<Games::Golf::TestSuite::run()>) allows the C<Games::Golf> object not to
 re-check already tested entries.
 
+You can pass a list of holes to test, and the object will test
+the solutions of these holes only.
+
+If no hole is given, then all holes are tested.
+
 =cut
 
 sub test {
-    my $self = shift;
-    foreach my $hole ( @{ $self->{holes} } ) {
+    my $self  = shift;
+    my @holes = @_ ? @_ : $self->hole_names;
+    foreach my $hole ( @holes ) {
         my @entries = grep { $_->hole eq $hole } values %{ $self->{entries} };
-        $self->{testers}{$hole}->run(@entries);
+        $self->{testers}{$hole}->run( @entries );
     }
 }
 
-=item test_all()
+=item dump( $path )
 
-Tests I<all> entries. Entries that were already tested will be tested
-again.
-
-=cut
-
-sub test_all {
-    my $self = shift;
-    foreach my $entry ( values %{ $self->{entries} } ) {
-        $self->{testers}{$entry->hole}->check( $entry );
-    }
-}
-
-=item dump( "/path/to/data" )
-
-Dumps all entries in a single file.
-
-Dies if there's a problem.
+Dump all entries in a single file.
 
 =cut
 
@@ -285,8 +314,12 @@ sub dump {
     $path or return;
 
     # Module we rely upon.
-    require Data::Dumper;
-    import Data::Dumper qw(Dumper);
+    eval { require Data::Dumper; };
+    if ( $@ ) {
+        carp "Module Data::Dumper required";
+        return;
+    }
+    import  Data::Dumper qw(Dumper);
 
     # Open file and dump.
     open DUMP, ">$path" or croak $!;
@@ -294,86 +327,63 @@ sub dump {
     close DUMP;
 }
 
-=item load( "/path/to/data" )
+=item load( $path )
 
-Reads a file with all previously recorded entries. See the C<dump>
+Read a file with all previously recorded entries. See the C<dump>
 method. This initiates the cache mechanism.
 
-Dies if there's a problem.
-
 =cut
 
 sub load {
     my ( $self, $path ) = @_;
-    $path or return;
-    $self->{entries} = do $path;
+    $self->{entries} = do $path or croak $!;
 }
 
-=item print_report( [*FH] )
-
-Prints the current leaderboard on STDOUT. If C<*FH> is supplied, use
-this filehandle instead.
-
-=cut
-
-# sub print_report {}
-
-=item mail_report( 'mail@adress.com', ... )
-
-Mails the leaderboard to the specified adresses.
-
-=cut
-
-# sub mail_report {}
-
-=item shell
-
-Interactive play for both a golfer and the arbiter.
-
 =back
 
-=cut
-
-# sub shel {}
-
 =head2 PRIVATE METHODS
 
 =over 4
 
-=item _defaults
+=item _parse_config_file(  )
 
-Returns a hash of allowed constructor attributes and their default values.
-
-=back
+Parse the config file, and fetch all attributes of the course. Create
+the testers for each of the holes.
 
 =cut
 
-{
-    my %_defaults = (
-         entries     => {},
-         deadline    => undef,
-         referees    => {},
-         holes       => [],
-         testers     => {},
-    );
-    
-    sub _defaults {
-        return %_defaults;
+sub _parse_config_file {
+    my $self = shift;
+
+    open CFG, "<$self->file" or croak "Can't open config file: $!";
+    my $content;
+    {
+        local $/;
+        $content = <CFG>;
     }
+    close CFG;
+
+    my $hole = ( $self->file =~ m!([^/]+)\.glf\z! );
+
+    # For now, the config file contains only the test-code for only
+    # one hole (multihole will be handled later).  Note that we
+    # prepare for multihole by using an anonymous hash, but the only
+    # value accepted by now is "hole".
+    $self->{testers}{$hole} = new Games::Golf::TestSuite( $content, "$hole.pl" );
 }
 
 1;
+
 __END__
 
+=head1 CONFIG FILE
+
+Once the course is launched, the referees are to provide a file which
+will describe the course.
+
+
 =head1 ENVIRONMENT VARIABLES
 
-=head2 PERL_GOLF_SMTP_SERVER
-
-This variable can be set to indicate which server is to be used to
-send solutions or report.
-
-!!FIXME!! What if the user wants to use a sendmail-based solution?
-
 =head2 PERL_GOLF_NICK
 
 Specifies the name to be printed on the leaderboard instead of the
@@ -381,8 +391,8 @@ mail adress of the golfer.
 
 =head2 PERL_GOLF_CATEGORY
 
-This one can be set to indicate if golfer is experienced (default,
-"veteran") or a newcomer that would set it to "beginner".
+This one can be set to indicate whether golfer is experienced
+(default, "veteran") or a newcomer ("beginner").
 
 =head1 FILES
 
